/*
 * Copyright 2012 Smallcrew, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package fr.smallcrew.security.web;

import fr.smallcrew.rest.exception.ResourceExceptionHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.web.authentication.rememberme.CookieTheftException;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.annotation.PostConstruct;
import javax.servlet.http.HttpServletRequest;

@Controller
@RequestMapping("/accessDenied")
public class AccessDeniedController {
  public static final String EXCEPTION = "SPRING_SECURITY_403_EXCEPTION";
  public static final String EXCEPTION_MESSAGE = "User is not logged";

  @Autowired
  private ResourceExceptionHandler exceptionHandler;

  @PostConstruct
  public void postConstruct() {
    exceptionHandler.addDefinition(AccessDeniedException.class, HttpStatus.FORBIDDEN);
    exceptionHandler.addDefinition(CookieTheftException.class, HttpStatus.CONFLICT);
    exceptionHandler.initialize();
  }

  private void throwException(HttpServletRequest request) {
    AccessDeniedException exception = (AccessDeniedException) request.getAttribute(EXCEPTION);
    if (exception == null) {
      exception = new AccessDeniedException(EXCEPTION_MESSAGE);
    }
    throw exception;
  }

  @RequestMapping(method = RequestMethod.GET)
  public void get(HttpServletRequest request) {
    throwException(request);
  }

  @RequestMapping(method = RequestMethod.POST)
  public void post(HttpServletRequest request) {
    throwException(request);
  }

  @RequestMapping(method = RequestMethod.DELETE)
  public void delete(HttpServletRequest request) {
    throwException(request);
  }

  @RequestMapping(method = RequestMethod.PUT)
  public void put(HttpServletRequest request) {
    throwException(request);
  }

  @RequestMapping(method = RequestMethod.HEAD)
  public void head(HttpServletRequest request) {
    throwException(request);
  }

  @RequestMapping(method = RequestMethod.OPTIONS)
  public void options(HttpServletRequest request) {
    throwException(request);
  }

  @RequestMapping(method = RequestMethod.TRACE)
  public void trace(HttpServletRequest request) {
    throwException(request);
  }
}
